@@ -18,10 +18,10 @@ from email.mime import multipart
 from email.mime import text
 import smtplib
 import time
+from urllib import parse
 
 from oslo_log import log as logging
 import requests
-import six
 
 from mistral import exceptions as exc
 from mistral import utils
@@ -46,6 +46,7 @@ class EchoAction(actions.Action):
         super(EchoAction, self).__init__()
 
         self.output = output
+
         try:
             self._delay = float(delay)
             self._delay = 0 if self._delay < 0 else self._delay
@@ -66,6 +67,15 @@ class EchoAction(actions.Action):
     def test(self, context):
         return 'Echo'
 
+    def __eq__(self, other):
+        if type(other) is not EchoAction:
+            return False
+
+        if self.output != other.output or self._delay != other._delay:
+            return False
+
+        return True
+
 
 class NoOpAction(actions.Action):
     """No-operation action.
@@ -106,6 +116,9 @@ class FailAction(actions.Action):
 
         self.error_data = error_data
 
+    def __repr__(self):
+        return 'Fail action'
+
     def run(self, context):
         LOG.info('Running fail action.')
 
@@ -171,7 +184,7 @@ class HTTPAction(actions.Action):
 
         if isinstance(headers, dict):
             for key, val in headers.items():
-                if isinstance(val, (six.integer_types, float)):
+                if isinstance(val, (int, float)):
                     headers[key] = str(val)
 
         if body and json:
@@ -212,7 +225,7 @@ class HTTPAction(actions.Action):
         )
 
         try:
-            url_data = six.moves.urllib.parse.urlsplit(self.url)
+            url_data = parse.urlsplit(self.url)
             if 'https' == url_data.scheme:
                 action_verify = self.verify
             else:
