@@ -11,35 +11,47 @@
 #    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
-import yaml
+
 
 from mistral.db.v2 import api as db_api
 from mistral import exceptions as exc
 from mistral.lang import parser as spec_parser
-from mistral import utils
+from mistral import services
+from mistral.utils import safe_yaml
 from mistral.workflow import states
+from mistral_lib import utils
 from oslo_log import log as logging
+from stevedore import extension
 
 
 STD_WF_PATH = 'resources/workflows'
 LOG = logging.getLogger(__name__)
 
 
-def register_standard_workflows(run_in_tx=True):
-    LOG.debug("Registering standard workflows...")
+def register_preinstalled_workflows(run_in_tx=True):
 
-    workflow_paths = utils.get_file_list(STD_WF_PATH)
+    extensions = extension.ExtensionManager(
+        namespace='mistral.preinstalled_workflows',
+        invoke_on_load=True
+    )
 
-    for wf_path in workflow_paths:
-        workflow_definition = open(wf_path).read()
+    for ext in extensions:
+        for wf_path in ext.obj:
+            register_workflow(run_in_tx, wf_path)
 
-        create_workflows(
-            workflow_definition,
-            scope='public',
-            is_system=True,
-            run_in_tx=run_in_tx,
-            namespace=''
-        )
+
+def register_workflow(run_in_tx, wf_path):
+    LOG.debug("Registering preinstalled workflow %s", wf_path)
+
+    workflow_definition = open(wf_path).read()
+
+    create_workflows(
+        workflow_definition,
+        scope='public',
+        is_system=True,
+        run_in_tx=run_in_tx,
+        namespace=''
+    )
 
 
 def _clear_system_workflow_db():
@@ -52,17 +64,18 @@ def sync_db():
     with db_api.transaction():
         _clear_system_workflow_db()
 
-        register_standard_workflows(run_in_tx=False)
+        register_preinstalled_workflows(run_in_tx=False)
 
 
 def create_workflows(definition, scope='private', is_system=False,
-                     run_in_tx=True, namespace=''):
+                     run_in_tx=True, namespace='', validate=True):
     LOG.debug("Creating workflows...")
 
     wf_list_spec = spec_parser.get_workflow_list_spec_from_yaml(
         definition,
-        validate=True
+        validate=services.is_validation_enabled(validate)
     )
+
     db_wfs = []
 
     if run_in_tx:
@@ -91,15 +104,20 @@ def create_workflows(definition, scope='private', is_system=False,
 def _append_all_workflows(definition, is_system, scope, namespace,
                           wf_list_spec, db_wfs):
     wfs = wf_list_spec.get_workflows()
-    wfs_yaml = yaml.load(definition)
+
+    wfs_yaml = safe_yaml.load(definition) if len(wfs) != 1 else None
+
     for wf_spec in wfs:
+        if len(wfs) != 1:
+            definition = _cut_wf_definition_from_all(
+                wfs_yaml,
+                wf_spec.get_name()
+            )
+
         db_wfs.append(
             _create_workflow(
                 wf_spec,
-                _cut_wf_definition_from_all(
-                    wfs_yaml,
-                    wf_spec.get_name()
-                ),
+                definition,
                 scope,
                 namespace,
                 is_system
@@ -108,12 +126,14 @@ def _append_all_workflows(definition, is_system, scope, namespace,
 
 
 def update_workflows(definition, scope='private', identifier=None,
-                     namespace=''):
-    LOG.debug("Updating workflows")
+                     namespace='', validate=True):
+    LOG.debug("Updating workflows...")
 
     wf_list_spec = spec_parser.get_workflow_list_spec_from_yaml(
-        definition, validate=True
+        definition,
+        validate=services.is_validation_enabled(validate)
     )
+
     wfs = wf_list_spec.get_workflows()
 
     if identifier and len(wfs) > 1:
@@ -125,19 +145,25 @@ def update_workflows(definition, scope='private', identifier=None,
 
     db_wfs = []
 
+    wfs_yaml = safe_yaml.load(definition) if len(wfs) != 1 else None
+
     with db_api.transaction():
-        wfs_yaml = yaml.load(definition)
         for wf_spec in wfs:
-            db_wfs.append(_update_workflow(
-                wf_spec,
-                _cut_wf_definition_from_all(
+            if len(wfs) != 1:
+                definition = _cut_wf_definition_from_all(
                     wfs_yaml,
                     wf_spec.get_name()
-                ),
-                scope,
-                namespace=namespace,
-                identifier=identifier
-            ))
+                )
+
+            db_wfs.append(
+                _update_workflow(
+                    wf_spec,
+                    definition,
+                    scope,
+                    namespace=namespace,
+                    identifier=identifier
+                )
+            )
 
     return db_wfs
 
@@ -189,7 +215,7 @@ def _update_workflow(wf_spec, definition, scope, identifier=None,
 
 
 def _cut_wf_definition_from_all(wfs_yaml, wf_name):
-    return yaml.dump({
+    return safe_yaml.dump({
         'version': wfs_yaml['version'],
         wf_name: wfs_yaml[wf_name]
-    })
+    }, sort_keys=False)
